==> layout-7.2.6-sources.jar: com/itextpdf/layout/renderer/BlockFormattingContextUtil.java <==
/*
    This file is part of the iText (R) project.
    Copyright (c) 1998-2024 Apryse Group NV
    Authors: Apryse Software.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License version 3
    as published by the Free Software Foundation with the addition of the
    following permission added to Section 15 as permitted in Section 7(a):
    FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY
    ITEXT GROUP. ITEXT GROUP DISCLAIMS THE WARRANTY OF NON INFRINGEMENT
    OF THIRD PARTY RIGHTS

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU Affero General Public License for more details.
    You should have received a copy of the GNU Affero General Public License
    along with this program; if not, see http://www.gnu.org/licenses or write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA, 02110-1301 USA, or download the license from the following URL:
    http://itextpdf.com/terms-of-use/

    The interactive user interfaces in modified source and object code versions
    of this program must display Appropriate Legal Notices, as required under
    Section 5 of the GNU Affero General Public License.

    In accordance with Section 7(b) of the GNU Affero General Public License,
    a covered work must retain the producer line in every PDF that is created
    or manipulated using iText.

    You can be released from the requirements of the license by purchasing
    a commercial license. Buying such a license is mandatory as soon as you
    develop commercial activities involving the iText software without
    disclosing the source code of your own applications.
    These activities include: offering paid services to customers as an ASP,
    serving PDFs on the fly in a web application, shipping iText with a closed
    source product.

    For more information, please contact iText Software Corp. at this
    address: sales@itextpdf.com
 */
package com.itextpdf.layout.renderer;

import com.itextpdf.io.util.NumberUtil;
import com.itextpdf.layout.layout.LayoutPosition;
import com.itextpdf.layout.properties.OverflowPropertyValue;
import com.itextpdf.layout.properties.Property;

/**
 * This class helps to identify whether we are dealing with a renderer that creates
 * a new "Block formatting context" in terms of CSS. Such renderers adhere to
 * specific rules of floating elements and margins collapse handling.
 * <p>
 * @see <a href="https://www.w3.org/TR/CSS21/visuren.html#block-formatting">https://www.w3.org/TR/CSS21/visuren.html#block-formatting</a>
 * @see <a href="https://developer.mozilla.org/en-US/docs/Web/Guide/CSS/Block_formatting_context">https://developer.mozilla.org/en-US/docs/Web/Guide/CSS/Block_formatting_context</a>
 */
public class BlockFormattingContextUtil {

    /**
     * Defines whether a renderer creates a new "Block formatting context" in terms of CSS.
     * <p>
     * See {@link BlockFormattingContextUtil} class description for more info.
     * @param renderer an {@link IRenderer} to be checked.
     * @return true if given renderer creates a new "Block formatting context" in terms of CSS, false otherwise.
     */
    public static boolean isRendererCreateBfc(IRenderer renderer) {
        return (renderer instanceof RootRenderer)
                || (renderer instanceof CellRenderer)
                || isInlineBlock(renderer)
                || renderer.getParent() instanceof FlexContainerRenderer
                || FloatingHelper.isRendererFloating(renderer)
                || isAbsolutePosition(renderer)
                || isFixedPosition(renderer)
                || isCaption(renderer)
                || AbstractRenderer.isOverflowProperty(OverflowPropertyValue.HIDDEN, renderer, Property.OVERFLOW_X)
                || AbstractRenderer.isOverflowProperty(OverflowPropertyValue.HIDDEN, renderer, Property.OVERFLOW_Y);
    }

    private static boolean isInlineBlock(IRenderer renderer) {
        return renderer.getParent() instanceof LineRenderer
                && (renderer instanceof BlockRenderer || renderer instanceof TableRenderer);
    }

    private static boolean isAbsolutePosition(IRenderer renderer) {
        Integer positioning = NumberUtil.asInteger(renderer.<Object>getProperty(Property.POSITION));
        return Integer.valueOf(LayoutPosition.ABSOLUTE).equals(positioning);
    }

    private static boolean isFixedPosition(IRenderer renderer) {
        Integer positioning = NumberUtil.asInteger(renderer.<Object>getProperty(Property.POSITION));
        return Integer.valueOf(LayoutPosition.FIXED).equals(positioning);
    }

    private static boolean isCaption(IRenderer renderer) {
        return renderer.getParent() instanceof TableRenderer
                && (renderer instanceof DivRenderer);
    }
}

==> layout-8.0.2-sources.jar: com/itextpdf/layout/renderer/BlockFormattingContextUtil.java <==
/*
    This file is part of the iText (R) project.
    Copyright (c) 1998-2023 Apryse Group NV
    Authors: Apryse Software.

    This program is offered under a commercial and under the AGPL license.
    For commercial licensing, contact us at https://itextpdf.com/sales.  For AGPL licensing, see below.

    AGPL licensing:
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.itextpdf.layout.renderer;

import com.itextpdf.io.util.NumberUtil;
import com.itextpdf.layout.layout.LayoutPosition;
import com.itextpdf.layout.properties.OverflowPropertyValue;
import com.itextpdf.layout.properties.Property;

/**
 * This class helps to identify whether we are dealing with a renderer that creates
 * a new "Block formatting context" in terms of CSS. Such renderers adhere to
 * specific rules of floating elements and margins collapse handling.
 * <p>
 * @see <a href="https://www.w3.org/TR/CSS21/visuren.html#block-formatting">https://www.w3.org/TR/CSS21/visuren.html#block-formatting</a>
 * @see <a href="https://developer.mozilla.org/en-US/docs/Web/Guide/CSS/Block_formatting_context">https://developer.mozilla.org/en-US/docs/Web/Guide/CSS/Block_formatting_context</a>
 */
public class BlockFormattingContextUtil {

    /**
     * Defines whether a renderer creates a new "Block formatting context" in terms of CSS.
     * <p>
     * See {@link BlockFormattingContextUtil} class description for more info.
     * @param renderer an {@link IRenderer} to be checked.
     * @return true if given renderer creates a new "Block formatting context" in terms of CSS, false otherwise.
     */
    public static boolean isRendererCreateBfc(IRenderer renderer) {
        return (renderer instanceof RootRenderer)
                || (renderer instanceof CellRenderer)
                || isInlineBlock(renderer)
                || renderer.getParent() instanceof FlexContainerRenderer
                || FloatingHelper.isRendererFloating(renderer)
                || isAbsolutePosition(renderer)
                || isFixedPosition(renderer)
                || isCaption(renderer)
                || AbstractRenderer.isOverflowProperty(OverflowPropertyValue.HIDDEN, renderer, Property.OVERFLOW_X)
                || AbstractRenderer.isOverflowProperty(OverflowPropertyValue.HIDDEN, renderer, Property.OVERFLOW_Y);
    }

    private static boolean isInlineBlock(IRenderer renderer) {
        return renderer.getParent() instanceof LineRenderer
                && (renderer instanceof BlockRenderer || renderer instanceof TableRenderer);
    }

    private static boolean isAbsolutePosition(IRenderer renderer) {
        Integer positioning = NumberUtil.asInteger(renderer.<Object>getProperty(Property.POSITION));
        return Integer.valueOf(LayoutPosition.ABSOLUTE).equals(positioning);
    }

    private static boolean isFixedPosition(IRenderer renderer) {
        Integer positioning = NumberUtil.asInteger(renderer.<Object>getProperty(Property.POSITION));
        return Integer.valueOf(LayoutPosition.FIXED).equals(positioning);
    }

    private static boolean isCaption(IRenderer renderer) {
        return renderer.getParent() instanceof TableRenderer
                && (renderer instanceof DivRenderer);
    }
}
